==> raspberryclient-1.0.0.dev4-py2.py3-none-any.whl: raspberryclient/websocket_communication.py <==
import json
import time
import logging
import websocket
from collections import deque

from .microphone_stream import MicrophoneStream
interspeech_sound = "./interspeech_sound.mp3"


class WsConstants:
    OPEN_AUDIO_STREAM = "{'type': 'InputAudioStreamOpen', 'sttConfig': {'language': 'cs', 'sampleRate': 16000}, 'appKey': 'brainquist', 'message': {'sender': '{1}', sessionId: '{2}'}}"
    CLOSE_AUDIO_STREAM = "{'type': 'InputAudioStreamClose', 'appKey': 'brainquist', 'message': {'sender': '{1}', sessionId: '{2}'}}"
    CANCEL_AUDIO_STREAM = "{'type': 'InputAudioStreamCancel', 'appKey': 'brainquist', 'message': {'sender': '{1}', sessionId: '{2}'}}"
    REQUIREMENTS = "{'type': 'Requirements', 'requirements': {'stt': 'true', 'tts': 'RequiredStreaming', 'returnSsml': 'true'}, 'appKey': 'brainquist'}"
    FORCE_PUSH = "{'type': 'Message', 'appKey': 'brainquist', 'message': {'sender': '{1}', sessionId: '{2}', 'items': [{'text': '$force_add'}]}}"
    SESSION_START = "{'type': 'SessionStarted', 'appKey': 'brainquist', 'message': {'sender': '{1}'}}"
    SESSION_END = "{'type': 'SessionEnded', 'appKey': 'brainquist', 'message': {'sender': '{1}', sessionId: '{2}'}}"


class WebsocketCommunication:
    websocket = None
    sending_microphone_bytes = False
    session_id = ""  # has to be set to "" to start new session

    def __init__(self, host, logger_name, device_id, on_alert_on, on_alert_off, on_session_start, on_session_end,
                 play_bytes, play_audio):
        """

        :param logger_name:
        :param device_id:
        :param on_alert_on: Turn the led on
        :param on_alert_off: Turn the led off
        :param on_session_start: Calling when the session id is set
        :param on_session_end: Calling when the server respond with SessionEnded
        :param play_bytes: Play audio - return False if the audio was interrupted
        """
        self.logger = logging.getLogger(logger_name)
        self.device_id = device_id
        # function
        self.on_alert_on = on_alert_on
        self.on_alert_off = on_alert_off
        self.on_session_start = on_session_start
        self.on_session_end = on_session_end
        self.play_bytes = play_bytes
        self.play_audio = play_audio

        self.websocket = websocket.WebSocketApp(host,
                                                on_open=self.on_open,
                                                on_message=self.on_message,
                                                on_error=self.on_error,
                                                on_close=self.on_close)

        self.session_end = False

    def on_message(self, message):
        if type(message) is str:
            self.logger.debug("### message ### - str")
            message = json.loads(message)
            self.logger.debug("Receiving: " + str(message))

            if message["type"] == "SessionStarted":
                self.logger.debug(
                    "Set session from '{}' to '{}'.".format(self.session_id, message["message"]["sessionId"]))
                self.session_id = message["message"]["sessionId"]
                self.on_session_start()
                self.session_end = False
            elif message["type"] == "InputAudioStreamOpen":
                self.play_audio(interspeech_sound)
                self.sending_microphone_bytes = True
            elif message["type"] == "Recognized":
                self.play_audio(interspeech_sound)
                self.sending_microphone_bytes = False
                self.send_message_ws(WsConstants.CLOSE_AUDIO_STREAM)
            elif message["type"] == "SessionEnded" or message["type"] == "Error":
                self.logger.debug("Set session from '{}' to ''.".format(self.session_id))
                self.session_id = ""
                self.on_session_end()
                self.sending_microphone_bytes = False
            elif message["type"] == "Message":
                if self.session_end: return
                self.send_message_ws(WsConstants.OPEN_AUDIO_STREAM)

        elif type(message) is bytes:
            if self.session_end: return

            self.logger.debug("### message ### - bytes")
            if self.play_bytes(message):  # if the whole audio is played
                self.logger.debug("Whole audio played!")
            else:  # if the audio is interrupted
                self.play_audio(interspeech_sound)
                self.send_message_ws(WsConstants.CANCEL_AUDIO_STREAM)
                self.send_message_ws(WsConstants.SESSION_END)

        else:
            self.logger.debug("I received message of type {}".format(type(message)))

    def on_error(self, error):
        self.logger.debug("### error ###")
        self.logger.debug(error)
        # self.play_audio("./no_internet.mp3")

    # is called even after on_error
    def on_close(self):
        self.logger.debug("### closed ###")
        self.session_id = ""
        self.on_session_end()

    def buffer(self, _audio_generator, max_len=3):
        """

        :param _audio_generator:
        :param max_len: Approximately 15 frames is 1 second
        :return:
        """
        _buffer = deque(maxlen=max_len)
        while True:
            try:
                _data = next(_audio_generator)
            except StopIteration:
                if len(_buffer) <= 0:
                    break
                _data = None

            if _data:
                _buffer.append(_data)

            if not self.sending_microphone_bytes:
                yield None
            elif self.sending_microphone_bytes:
                yield _buffer.popleft()

    def run(self, *args):
        with MicrophoneStream() as stream:
            audio_generator = stream.generator()
            for data in self.buffer(audio_generator):
                if self.sending_microphone_bytes:
                    if data:
                        self.websocket.send(data, opcode=websocket.ABNF.OPCODE_BINARY)
                        self.on_alert_on()
                else:
                    self.on_alert_off()

    def on_open(self):
        self.send_message_ws(WsConstants.REQUIREMENTS)

    def session_start(self):
        # self.send_message_ws(WsConstants.SESSION_START)
        while self.is_session_started():
            self.logger.debug("sessionId:{}".format(self.session_id))
            time.sleep(0.1)

    def is_session_started(self):
        return self.session_id != ""

    def force_push(self):
        self.send_message_ws(WsConstants.FORCE_PUSH)

    def force_stop(self):
        self.session_end = True
        self.play_audio(interspeech_sound)
        self.send_message_ws(WsConstants.CANCEL_AUDIO_STREAM)
        self.send_message_ws(WsConstants.SESSION_END)

    def send_message_ws(self, msg_string):
        """
        {1} is replaced with UserId
        {2} is replaced with SessionId
        :param msg_string:
        :return:
        """
        self.logger.debug("Sending: " + msg_string.replace("{1}", self.device_id).replace("{2}", self.session_id))
        self.websocket.send(msg_string.replace("{1}", self.device_id).replace("{2}", self.session_id))

==> raspberryclient-1.0.1-py2.py3-none-any.whl: raspberryclient/websocket_communication.py <==
import json
import time
import logging
import websocket
from collections import deque

from .microphone_stream import MicrophoneStream
interspeech_sound = "./interspeech_sound.mp3"


class WsConstants:
    OPEN_AUDIO_STREAM = "{\"type\": \"InputAudioStreamOpen\", \"appKey\": \"{3}\", \"message\": {\"sender\": \"{1}\", \"sessionId\": \"{2}\"}}"
    CLOSE_AUDIO_STREAM = "{\"type\": \"InputAudioStreamClose\", \"appKey\": \"{3}\", \"message\": {\"sender\": \"{1}\", \"sessionId\": \"{2}\"}}"
    CANCEL_AUDIO_STREAM = "{\"type\": \"InputAudioStreamCancel\", \"appKey\": \"{3}\", \"message\": {\"sender\": \"{1}\", \"sessionId\": \"{2}\"}}"
    REQUIREMENTS = "{\"type\": \"Requirements\", \"requirements\": {\"stt\": \"true\", \"tts\": \"RequiredStreaming\", \"returnSsml\": \"true\"}, \"appKey\": \"{3}\"}"
    FORCE_PUSH = "{\"type\": \"Message\", \"appKey\": \"{3}\", \"message\": {\"sender\": \"{1}\", \"sessionId\": \"{2}\", \"items\": [{\"text\": \"$intro\"}]}}"
    SESSION_START = "{\"type\": \"SessionStarted\", \"appKey\": \"{3}\", \"message\": {\"sender\": \"{1}\"}}"
    SESSION_END = "{\"type\": \"SessionEnded\", \"appKey\": \"{3}\", \"message\": {\"sender\": \"{1}\", \"sessionId\": \"{2}\"}}"

class WebsocketCommunication:
    websocket = None
    sending_microphone_bytes = False
    session_id = ""  # has to be set to "" to start new session

    def __init__(self, host, logger_name, device_id, app_key, led_on, led_off, on_session_start, on_session_end,
                 play_bytes, play_audio):
        """

        :param logger_name:
        :param device_id:
        :param led_on: Turn the led on
        :param led_off: Turn the led off
        :param on_session_start: Calling when the session id is set
        :param on_session_end: Calling when the server respond with SessionEnded
        :param play_bytes: Play audio - return False if the audio was interrupted
        """
        self.logger = logging.getLogger(logger_name)
        self.device_id = device_id
        self.app_key = app_key
        # function
        self.led_on = led_on
        self.led_off = led_off
        self.on_session_start = on_session_start
        self.on_session_end = on_session_end
        self.play_bytes = play_bytes
        self.play_audio = play_audio

        self.websocket = websocket.WebSocketApp(host,
                                                on_open=self.on_open,
                                                on_message=self.on_message,
                                                on_error=self.on_error,
                                                on_close=self.on_close)

        self.force_end = False
        self.recording = False

    def on_message(self, message):
        if type(message) is str:
            self.logger.debug("### message ### - str")
            message = json.loads(message)
            self.logger.debug("Receiving: " + str(message))

            if message["type"] == "SessionStarted":
                self.logger.debug(
                    "Set session from \"{}\" to \"{}\".".format(self.session_id, message["message"]["sessionId"]))
                self.session_id = message["message"]["sessionId"]
                self.on_session_start()
                self.force_end = False
            elif message["type"] == "InputAudioStreamOpen":
                if self.is_session_started():
                    self.sending_microphone_bytes = True
            elif message["type"] == "Recognized":
                self.play_audio(interspeech_sound)
                self.sending_microphone_bytes = False
                self.send_message_ws(WsConstants.CLOSE_AUDIO_STREAM)
            elif message["type"] == "SessionEnded" or message["type"] == "Error":
                self.logger.debug("Set session from \"{}\" to \"\".".format(self.session_id))
                self.session_id = ""
                self.on_session_end()
                self.sending_microphone_bytes = False
                self.recording = False
            elif message["type"] == "Message":
                if self.force_end:
                    self.play_audio(interspeech_sound)
                    time.sleep(1)
                    self.play_audio(interspeech_sound)
                    return

                self.send_message_ws(WsConstants.OPEN_AUDIO_STREAM)
                self.play_audio(interspeech_sound)
                self.recording = True
                self.led_on()

        elif type(message) is bytes:
            if self.force_end:
                self.on_session_end()
                return
            self.recording = False

            self.logger.debug("### message ### - bytes")
            if self.play_bytes(message):  # if the whole audio is played
                self.logger.debug("Whole audio played!")
            else:  # if the audio is interrupted
                self.play_audio(interspeech_sound)
                self.send_message_ws(WsConstants.CANCEL_AUDIO_STREAM)
                self.send_message_ws(WsConstants.SESSION_END)

        else:
            self.logger.debug("I received message of type {}".format(type(message)))

    def on_error(self, error):
        self.logger.debug("### error ###")
        self.logger.debug(error)
        # self.play_audio("./no_internet.mp3")

    # is called even after on_error
    def on_close(self):
        self.logger.debug("### closed ###")
        self.session_id = ""
        self.on_session_end()

    def buffer(self, _audio_generator, max_len=30):
        """

        :param _audio_generator:
        :param max_len: Approximately 15 frames is 1 second
        :return:
        """
        _buffer = deque(maxlen=max_len)
        while True:
            try:
                _data = next(_audio_generator)
            except StopIteration:
                _data = None

            if _data:
                if self.recording:
                    _buffer.append(_data)
                else:
                    _buffer.clear()

            if not self.sending_microphone_bytes:
                yield None
            elif self.sending_microphone_bytes:
                yield _buffer.popleft()

    def run(self, *args):
        with MicrophoneStream() as stream:
            audio_generator = stream.generator()
            for data in self.buffer(audio_generator):
                if data:
                    self.websocket.send(data, opcode=websocket.ABNF.OPCODE_BINARY)
                else:
                    self.led_off()
                    time.sleep(0.3)

    def on_open(self):
        self.send_message_ws(WsConstants.REQUIREMENTS)

    def session_start(self):
        # self.send_message_ws(WsConstants.SESSION_START)
        while self.is_session_started():
            self.logger.debug("sessionId:{}".format(self.session_id))
            time.sleep(0.1)

    def is_session_started(self):
        return self.session_id != ""

    def force_push(self):
        self.send_message_ws(WsConstants.FORCE_PUSH)

    def force_stop(self):
        self.force_end = True
        self.send_message_ws(WsConstants.CANCEL_AUDIO_STREAM)
        self.send_message_ws(WsConstants.SESSION_END)

    def send_message_ws(self, msg_string):
        """
        {1} is replaced with UserId
        {2} is replaced with SessionId
        :param msg_string:
        :return:
        """
        str_to_send = msg_string.replace("{1}", self.device_id).replace("{2}", self.session_id).replace("{3}", self.app_key)
        self.logger.debug(str_to_send)
        self.websocket.send(str_to_send)
